%------------------------------------------------------------------------
% Routine     : susim(FILE)
% Description : Saves configuration to FILE if exists, prompt for file
%               if not. 
% 
% Language    : Matlab 4.1
% Written by  : Vebjorn Berre
%
% Version no. : 1.0
%             : 1.1 TKL 15.07.93 Added directory possibilities for user
%             : 1.2 TKL 03.09.93 Added "analysis" variable
%             : 1.3 TKL 22.09.93 Removed "analysis" variable
%             : 1.4 FT  19.01.94 Added cnf-variables for layered medium
%                                Uses 'filename' as filename if it exists
%             : 2.0 FT  15.02.94 Added user-defined media and pulse (if
%                                 exist). Added elecdelr/elecdelt
%             : 2.1 FT  20.03.94 Removed 'linjeparametre'
%             : 2.2 FT  23.03.94 Corrected bug
%             : 2.3 FT  02.05.94 Saving phase_err_ud if it exists
%             : 2.4 BJE 21.11.94 Changed text to fit smaller windows
%                                Added *.cnf filenames
%               2.5 BJE 30.11.94 Added path separator
%               2.6 SH   7.12.94 Removed trailing d from pwd
%               2.7 LO  22.12.94 Global variable to layer modified
%               2.8 SH   3. 1.95 Check for existence of max_resp, min_resp
%               2.9 SH  19.01.95 Added comment and usr_cnfdir in file header
%               3.0 BJE 01.05.95 Removed path separator, exists already!
%                                d -> separator, script -> function
%                                added save as 'default.cnf'
%               3.1 SH  16.05.95 changed 'default.cnf' to 'startup.cnf'
%               3.2 SH  20.05.96 Made comment an output
%               3.3 SH  06.09.96 Removed code for writing global variables
%               3.4 SH  17.09.96 Removed variables from saving, added centers
%                                => More compact fileformat for *.cnf files
%               3.5 SH  04.06.97 Matlab 5: saves files in matlab4 format
%               7.0 SH  08.02.05 extra ''s to deal with spaces in filenames
%               

% Called by   : usimcnf.m, usimquit.m
% Calling     : none
%------------------------------------------------------------------------

function [comment]=susim(FILE, excitation, observasjon, media, transducer, ...
         flagg, ...
         option, x, y, z, t, elem_pts, elecdelr, elecdelt, comment, ...
         pvec, plength, phase_err_ud, amp_ud, ...
         array_resp, max_resp, min_resp, ...
         Elec, Mech, usr_cnfdir, USER_ULTRASIMHOME, separator,centers)
  
  
%
% init
%
  write_ok=1;   % boolean var
  
  
%
% get file to save if FILE is empty
%
  if isempty(FILE)
      
      tmp=menu35('SAVE configuration to',...
             'Working directory','Your own [cnf] directory','Your own local startup.cnf');
      
            
%	 
% default
%
      if isempty(tmp) 
          tmp=2;
      end
      
%
% assign desired directory
%
      if tmp==2
          cnfdir=[usr_cnfdir];
        elseif tmp==1
          cnfdir=[pwd];
      end

      
      if tmp==3
	  FILE=[USER_ULTRASIMHOME,separator,'startup.cnf'];
        else
          
%
% GET FILENAME ... display current config files in cnfdir
%
          disp('  ')
          disp(['Current files in ',cnfdir])
          disp('  ')
          tmp_dir=pwd;
% extra '' to deal with spaces in filenames          
          eval(['cd ''',cnfdir,'''']);
          eval(['dir *.cnf']);            % recommended extension
          eval(['dir *cnf.mat']);
          eval(['cd ''',tmp_dir,'''']);
          clear tmp_dir

	  disp('  ')
          disp('If not filename is of type ''*.cnf'' or ''*cnf.mat'',')
          disp('an ''.cnf'' extension is added to the filename!')
          disp('  ')
          FILE = input('-> Save file as [.cnf] : ','s');
          
          if isempty(FILE)
              write_ok=0;
            else
              
%
% ... and new comment if input
%
              if exist('comment')
                  disp(' ')
                  disp(' ')
                  disp(' ')
                  disp('Current comment : ')
                  disp(' ');
                  disp(comment);    
              end
              
              tmp=input('-> Enter comment (return for keep old) : ','s');
              
              if ~isempty(tmp) 
                  comment = tmp;
              end
              
%
% ... append '.cnf' extension if necessary
%
              if isempty(findstr(FILE,'cnf'))
                  FILE=[FILE,'.cnf'];
              end
              
%
% add path and system dependent separator 
%
              FILE=[cnfdir,separator,FILE];
              
%
% ... check if file already exists, if it does, ask to overwrite
%
              
              if exist(FILE)
                  disp('  ')
                  disp('WARNING: File already exists.')
                  disp('  ')
                  tmp_overwrite = input('-> Do you want to overwrite [y/n] ? ','s');
                  if all(tmp_overwrite ~= 'y')
                      write_ok=0;
                  end
              end

	      
          end    % else write_ok                
          
      end    % else GET FILENAME    
      
  end    % GET FILENAME, much nesting !!


%
% Update comment and save configuration if write_ok
%
  
  if ~(write_ok)
      disp('  ')
      disp('Did not write file.') % default
      
    else      
      
%
% make up a string of general variables to save
% ... all these variables are at least initialized in usiminit.
%
      tmp_var = [' excitation observasjon media transducer flagg option ',...
             'elecdelr elecdelt comment '];
% 
% save elem_pts and centers for rectangular arrays in order to save sparse array layout
%
      if flagg(2) == 1, tmp_var = [tmp_var,' elem_pts centers']; end;
%
      if ~isempty(pvec) tmp_var = [tmp_var ' pvec']; end;
      if ~isempty(plength) tmp_var = [tmp_var ' plength']; end;
      if ~isempty(phase_err_ud) tmp_var = [tmp_var ' phase_err_ud']; end;
      if ~isempty(amp_ud) tmp_var = [tmp_var ' amp_ud']; end;      
% 
% removed from save:
% x,y,z,t, array_resp, max_resp, min_resp
%
%             'x y z t elem_pts elecdelr elecdelt comment ',...
%             'pvec plength phase_err_ud amp_ud ',...
%             'array_resp max_resp min_resp '];
      
      
%
% and some global variables - Get rid of global variables in the future ??? 
% removed 6.9.96 SH
      
%      if flagg(3)==2
%          global abrflagg fixeddepth pvector
%          if abrflagg~=[]
%              tmp_var = [tmp_var,...
%                     ' abrflagg fixeddepth pvector'];
%          end
          ind = find(media(:,3)==5);
          if ~isempty(ind)
              for n = 1:length(ind)
                  eval(['global bound',int2str(media(ind(n),4))]);
                  tmp_var = [tmp_var,' ','bound',int2str(media(ind(n),4))];
              end
          end
      end
      
%
% save all variables in tmp_var
%
      vers = version;
      if vers(1)=='5'
	eval(['save ''' FILE,'''', tmp_var ' -v4']);
      else
 % ''s         
	eval(['save ''' FILE,'''', tmp_var]);
      end
      disp('  ')
      disp(['Wrote file as ' FILE ' in ver 4 format'])
      

  
  
  clear tmp_var tmp_overwrite tmp write_ok
  
  fprintf('\n\nFinished!\n');
  




