%------------------------------------------------------------------------
% Routine       : t3usim       
%
% Description   : Configuring transducer-parameters
%			   ANNULAR ARRAY
%
% Language      : Matlab 4.1
% Written by    : Vebjorn Berre, IBT
% 
% Version no.   : 1.0 VEB 27.10.93 First Version
% Version no.	: 2.0 VEB 28.03.93 
% Version no.	: 3.0 SH  16.04.93 
% Version no.   : 3.1 FT  27.09.93 Created menu for selection of trans-
%				   ducer type. Added User-Defined Width
%               : 3.2 FT  09.02.93 Parameter change depend on flagg(3)
%               : 3.3 FT  23.02.94 Removed disp('EXIT = "x"')
%               : 3.4 SH   6.12.94 Changed call: t3usimtxt to t3usimtx
%               : 3.5 SH  03.06.96 centers as output
%               : 3.6 HJG 29.01.96 Bessel-Beam is added to the menu & 
%               : 3.7 SH  01.10.96 centers=[] to save memory
%                                  it calls :vinput 
%                 3.8 SH   9.10.96 put Bessel beam order in transducer(13) 
%                                      rotation angle in transducer(14)
%                                      rearranged order of tests
%                 3.9 SH  11.11.96 Improved user interface for Bessel
%                 4.0 SH  06.02.97 Conical transducer
%                 4.1 SH  25.03.99 order = 0 for options 5, 6
%                                  changed max value for alpha
%                                  allows evanescent Bessel waves (large alpha)
%                 5.0 SH 18.05.05  removed if test for to avoid error message
%
% Called by	: usimcnf
% Calling       : t3usimtx
%------------------------------------------------------------------------

function [transducer,elem_pts, centers]=t3usim(transducer,flagg,excitation,media)
global amp_ud
fid=1;
lambda=media(1,2)/excitation(1);
if length(flagg)>4
  if flagg(5)==1
    lambda_local=1;
  elseif flagg(5)==2
    lambda_local=media(1,2)/excitation(1)*1e3;
  end
else
  lambda_local=1;
end

tmp=size(transducer);
if tmp(2)<18
  tmp=transducer;
  transducer=zeros([1,18]);
  transducer([1:1:length(tmp)])=tmp;
end

com1=0;
while com1~='x'
  clc
  t3usimtx;
  
  disp('	CHANGE 		= "number"')
  com1 = input('-> Decision (<CR> = exit): ','s');
  if isempty(com1) com1 = 'x'; end
  if length(com1)==1 com1=['0',com1];end
  if length(com1)>2
    com1 = '00'
  end
    
  % changing of parameters
  
  if com1=='01'
    temp=input('Transducer diameter 		D = ');
    if ~isempty(temp) transducer(1)=temp*lambda_local*1e-3; end
  
  elseif com1=='02'
    temp=input('# elements		                N  =  ');
    if ~isempty(temp) transducer(2)=temp; end
    
  elseif com1=='03'
    if flagg(3)~=2
      temp=input('# points on transducer surface	P = ');
    else
      temp=input('# rays from transducer surface	P = ');
    end
    if ~isempty(temp) transducer(3)=temp; end

  
    elseif com1=='18'
    temp = menu35_0('Choose Transducer Type', ...
           'Equal Area','Equal Width','Circular','User-Defined Width', ...
           'Exact Bessel(n) beam (1 element)', ...
           'Bessel(0) equal-area transducer (N elements)',... 
           'Bessel(0) transducer (N elements)', ...
           '4th derivatives bowtie Bessel beam', ...
           '10th Derivatives bowtie Bessel beam ', ... 
           'Exact non-rotating Bessel(n) beam (1 element)', ...
	   'Conical transducer (equal-area)');
    %---------------------Bessel------------------------------ 
    order = transducer(12);	
    scl_factor = transducer(13);
    phi = transducer(14);


    if (temp==4) 			% nth-order Bessel beam (1 element)
      disp(' ')
       disp('Make sure that Apodization is User-Defined (points) !!!')

       scl_factor=vinput(['Scaling factor [m](',num2str(0),'...',num2str(2*pi/lambda),')'],0,100*2*pi/lambda,scl_factor); 
       order=vinput('Order of Bessel function (>= 0)',0,1000,order);
       transducer(2) = 1;
       phi = 0;
	
    elseif (temp== 5)%  Equal-area 0th-order Bessel beam (N elements)
       disp(' ')
       disp('Make sure that Apodization is User-Defined (elements) !!!')
       disp('Transducer aperture D, and # elements N must be choosen')
       disp('before choice of transducer type ......')
       scl_factor=vinput(['Scaling factor [m](',num2str(0),'...',num2str(2*pi/lambda),')'],0,2*10*pi/lambda,scl_factor);  
       order=0;
       disp(' ')
       phi = 0;
       [transducer(2)]=lulu(scl_factor,transducer(2),transducer(1),'y');  
   
     elseif (temp== 6)%  0th-order Bessel transducer(N elements)
       disp(' ')
       disp('Make sure that Apodization is User-Defined (elements) !!!')
       disp('Transducer aperture D, and # elements N must be choosen')
       disp('before choice of transducer type ......')
       scl_factor=vinput(['Scaling factor [m](',num2str(0),'...',num2str(2*pi/lambda),')'],0,2*10*pi/lambda,scl_factor);  
       order=0;
       disp(' ')
       phi = 0;
%      disp('WAIT CALCULATING amp_ud.......')
       [transducer(2)]=lulu(scl_factor,transducer(2),transducer(1),'n');  
 
       %----------------------nth-order Bessel beam (N elements)  

    elseif (temp==7) 			% 4th derivatives bowtie Bessel beam
       scl_factor=vinput(['Scaling factor [m](',num2str(0),'...',num2str(2*pi/lambda),')'],0,2*10*pi/lambda,scl_factor); 
       transducer(2) = 1;
       phi = 0;

    elseif (temp==8)%  10th Derivatives bowtie Bessel beam
       scl_factor=vinput(['Scaling factor [m](',num2str(0),'...',num2str(2*pi/lambda),')'],0,2*10*pi/lambda,scl_factor); 
       transducer(2) = 1;
       phi = 0;
 
   elseif (temp == 9)% nth-order non-rotating Bessel beam (1 element)
       disp(' ')
       disp('Make sure that Apodization is User-Defined (points) !!!')
       scl_factor=vinput(['Scaling factor [m](',num2str(0),'...',num2str(2*pi/lambda),')'],0,2*10*pi/lambda,scl_factor); 
       order=vinput('Order of Bessel function (>= 0)',0,1000,order);
       phi=vinput('Rotation angle [deg] = ',-360,360,phi) %  Rotation angle
       transducer(2) = 1;
    
    end

    if ~isempty(temp) transducer(18)=temp; end
     
  elseif com1=='09'
    temp=input('Fixed focus 			F = ');
    if ~isempty(temp) transducer(9)=temp*lambda_local*1e-3; end

  elseif com1=='10' & flagg(3)==2
    temp=input('Rotation angle [deg]		theta = ');
    if ~isempty(temp) transducer(10)=temp*pi/180; end
%
% conical beam
%

  elseif com1=='12' & transducer(18)==10
       transducer(12) = (pi/180)*vinput('Axicon angle [deg] = ', ...
	   -90,90,180*transducer(12)/pi);
       
  elseif com1=='15' & flagg(3)==2
    temp=input('Offset	[mm]			dz = ');
    if ~isempty(temp) transducer(15)=temp*1e-3; end


%--------------------------------------------------------------------------
  end
  if (transducer(18) >=4 & transducer(18) <=9),
    if exist('order'), transducer(12) = order; end % order of Bessel-beam
    if exist('scl_factor'), transducer(13) = scl_factor; end
    if exist('phi'), transducer(14) = phi; end % rotation angle of Bessel-beam
   end
end 				


% if test removed 18.05.2005, SH:
%if flagg(3)~=2
  [elem_pts] = annular(transducer, excitation, media);
  centers = []; % no separate calculation of element centers done so far!
%end

%------------Bessel Beam----------22.01.1996

if ( transducer(18)==4|transducer(18)==7|transducer(18)==8 | transducer(18)==9)

  transducer(2) = 1;
  if (transducer(18) == 4),
    amp_ud=zeros(size(elem_pts(1:2,:)));
  else
    amp_ud=zeros(size(elem_pts(1:3,:)));
  end
  rho=sqrt(elem_pts(1,:).^2+elem_pts(2,:).^2); % m (rho is radial coordinate)


  %-----------------nth-order Besselbeam (1 element)--------     
  if ( temp== 4)
    jbes= (bessel(order,abs(rho)*scl_factor)); % Bessel Beam (nth-order)
    amp_ud(1,:)=jbes;
    amp_ud(2,:)=rho.*1000; 		% mm

    %-----------------nth-order non-rotating Bessel beam (1 element)------  
  elseif (temp==9)
    if (order == 0)
      j0=(bessel(0,abs(rho)*scl_factor)); % Bessel Beam (nth-rotating)
      amp_ud(1,:)=j0;
    else 
      fi=atan2((elem_pts(1,:)),(elem_pts(2,:)));
      jbes2=(bessel(order,abs(rho)*scl_factor)).*(cos(order*(fi-phi*pi/180))); 
      amp_ud(1,:)=jbes2;
    end
    amp_ud(2,:)=1000*elem_pts(1,:); 		% x
    amp_ud(3,:)=1000*elem_pts(2,:);             % y

    %----------------------4th derivatives Bowtie Bessel baem------------------
  elseif ( temp==7)
    a=find(rho==0);
    j0=(bessel(0,abs(rho)*scl_factor)); % Bessel Beam (nth-rotating)
    j1=(bessel(1,abs(rho)*scl_factor));
    alf=scl_factor;
    y=elem_pts(2,:);
    x1=-(24*alf.^2*y.^4)./rho.^6;
    x1(a)=0;
    x2=(alf.^2*y.^2.*(24+alf^2*y.^2))./rho.^4;
    x2(a)=0;
    x3=3*alf.^2./rho.^2;
    x3(a)=0;
    x4=(48*alf*y.^4)./rho.^7;
    x4(a)=0;
    x5=(8*alf*y.^2.*(6+alf.^2*y.^2))./rho.^5;
    x5(a)=0;
    x6=(6*alf*(1+alf.^2*y.^2))./rho.^3;
    x6(a)=0;
    j4=8/(3*alf^4)*((x1+x2-x3).*j0+(x4-x5+x6).*j1);
    j4(a)=1;
    amp_ud(1,:)=j4; 
    amp_ud(2,:)=1000*elem_pts(1,:);     % x
    amp_ud(3,:)=1000*elem_pts(2,:);


    %-----------------------10th derivatives bowtie Bessel beam--------    
  elseif (temp==8)
    a=find(rho==0);
    j0=(bessel(0,abs(rho)*scl_factor)); % Bessel Beam (nth-rotating)
    j1=(bessel(1,abs(rho)*scl_factor));
    alf=scl_factor;
    y=elem_pts(2,:);
    xx1=-(92897280*alf.^2*y.^10./rho.^18);
    xx1(a)=0;
    xx2=1290240*alf.^2*y.^8.*(180+7*alf.^2*y.^2)./rho.^16;
    xx2(a)=0;
    xx3=201600*alf.^2*y.^6.*(1008+108*alf.^2*y.^2+alf.^4*y.^4)./rho.^14;
    xx3(a)=0;
    xx4=1200*alf.^2*y.^4.*(60480+15120*alf.^2*y.^2+360*alf.^4*y.^4+alf.^6*y.^6)./rho.^12;
    xx4(a)=0;
    xx5=alf.^2*y.^2.*(9072000+6048000*alf.^2*y.^2+302400*alf.^4*y.^4+1800*alf.^6*y.^6+alf.^8*y.^8)./rho.^10;
    xx5(a)=0;
    xx6=630*alf.^2*(288+1080*alf.^2*y.^2+120*alf.^4*y.^4+alf.^6*y.^6)./rho.^8;
    xx6(a)=0;
    xx7=945*alf.^4*(12+5*alf.^2*y.^2)./rho.^6;
    xx7(a)=0;
    xx8=185794560*alf*y.^10./rho.^19;
    xx8(a)=0;
    xx9=10321920*alf*y.^8.*(45+4*alf.^2*y.^2)./rho.^17;
    xx9(a)=0;
    xx10=1693440*alf*y.^6.*(240+60*alf.^2*y.^2+alf.^4*y.^4)./rho.^15;
    xx10(a)=0;
    xx11=9600*alf*y.^4.*(15120+9072*alf.^2*y.^2+405*alf.^4*y.^4+2*alf.^6*y.^6)./rho.^13;
    xx11(a)=0;
    xx12=50*alf*y.^2.*(362880+604800*alf.^2*y.^2+60480*alf.^4*y.^4+720*alf.^6*y.^6+alf.^8*y.^8)./rho.^11;
    xx12(a)=0;
    xx13=45*alf*(8064+80640*alf.^2*y.^2+20160*alf.^4*y.^4+448*alf.^6*y.^6+alf.^8*y.^8)./rho.^9;
    xx13(a)=0;
    xx14=630*alf.^3*(108+135*alf.^2*y.^2+5*alf.^4*y.^4)./rho.^7;
    xx14(a)=0;
    xx15=945*alf.^5./rho.^5;
    xx15(a)=0;
    j10=-256/((63*alf.^10))*((xx1+xx2-xx3+xx4-xx5+xx6-xx7).*j0+(xx8-xx9+xx10-xx11+xx12-xx13+xx14-xx15).*j1);
    j10(a)=1;
    amp_ud(1,:)=j10; 
    amp_ud(2,:)=1000*elem_pts(1,:); 		% x
    amp_ud(3,:)=1000*elem_pts(2,:);
  end
end








