%---------------------------------------------------------------
% Routine     : peplot3d.m ( PE -  Plot in 3D )
% 
% Description : Plots the results from the PE- analysis 
%               of Volumes in 'plotfig' window
%
% Language    : Matlab V4.1
%
% Written by  : Department of Informatics, Kapila Epasinghe
%
% Version no. : 1.0 KE 20.01.96 First version
%               1.1 KE 01.10.96 Added matrix 'visualize'
%               1.2 KE 03.02.97 Gives option to change plotoptions
%               1.3 SH 26.01.98 Removed transpose of resp 
%                   SH 15.03.01 Made it run under Matlab6
%               7.0 SH 09.02.05 '' to deal with spaces in filenames
%
% Called by   : usimcnf
% Calling     : cleanscr,setcomap
%---------------------------------------------------------------


function peplot3d(resp, max_resp, min_resp, excitation,transducer, media, ...
	observation, option, flagg, x,y,z,t, config, visualize, ...
	plotfig, plotfigOrigHandle, usr_resdir)

% make up a vector of analysis times

if option(1) == 'f'
   delta_t=1/(1e3*excitation(14)*observation(15));
   TimeStepVector = [observation(4):delta_t:observation(8)];
   TLen = length(TimeStepVector);
else
   TLen = 1;
end


% INITIALIZATION

clc
cleanscr(plotfig, plotfigOrigHandle);
figure(plotfig);
MeshCont = 'Mesh';
setcomap(plotfig,5);

if (flagg(2) == 1),
   n_elem = transducer(2)*transducer(6);
else
   n_elem = transducer(2);
end


% Set only envelope option for the time being
if (length(visualize)<=1)
   clc
   disp('Set the fillowing plot options: ');
   disp('  ');

   plot_type = input('-> envelope / rf [e/r] ? ','s');
   if all(plot_type ~= 'r')
      plot_type = 'e';   % default
   end

   if all(plot_type == 'r')
      plot_option = 'lin';
   else
      plot_option=input('-> linear /logarithmic [lin/log] ? ','s');
      if all(plot_option ~= 'log')
         plot_option = 'lin';   % default
      end
   end
else
   fprintf('Plot Options set as :\n');
   fprintf('\t 1. ');
   if visualize(1)=='e'
      fprintf('Envelope\n');
   else
      fprintf('rf\n');
   end
   fprintf('\t 2. ');
   if visualize(2:4)=='lin'
      fprintf('Linear\n');
   else
      fprintf('Logarithmic\n');
   end
   fprintf('\t 3. Colormap %s\n',visualize(5:10));
   com=input('-> Accept ??? [y/n] : ','s');
   if com=='n'
      visualize=setvisu(option,flagg,visualize);
   end;
   plot_type=visualize(1);
   plot_option=visualize(2:4);
end

% Does not set view option...! Not very actual on a 3D visualization.

tmp_view=3;
vtxt = ['   View: 3D default'];


% Set the text variables

[Xtxt, Ytxt, Ztxt, Ttxt1, Ttxt2, Ttxt3] = ...
	petxt(excitation, media, transducer, option, flagg, z,t, ...
	plot_type,plot_option, MeshCont);

% Set axis, 3rd axis either time analysis or distance 

if option(1) == '3' | option(1) == 'f'
   Xx=[observation(1):1/(1e3*observation(15)):observation(5)]*1e3;
   Yy=[observation(2):1/(1e3*observation(15)):observation(6)]*1e3;
   if option(3) == 't'
      Zz=[observation(4):1/(1e6*observation(15)):observation(8)]*1e6;
   else
      Zz=[observation(3):1/(1e3*observation(15)):observation(7)]*1e3;
   end
end
% Get current axes position modify it (plot axes)

figure(plotfig);
handle1 = gca;
axis('off');
v1 = get(handle1,'Position');
v1(4) = v1(4) - 0.1;
set(handle1,'Position',v1);

% Make another axes for text (text axes)

v2= v1;
v2(1) = v2(1) - 0.1;
v2(2) = v2(2) + v2(4) + 0.05;
v2(4) = 0.1;
handle2= axes('Position',v2);
axis('off');


% Use of max min z not needed???
rmovie='n';
slcx=plt3d(rmovie,'X',Xx,Xx(length(Xx)),observation);
slcy=plt3d(rmovie,'Y',Yy,Yy(1),observation);
if option(1)=='3'
   rmovie=input('-> Range Movie on Third axis ?? [y/n] :','s');
end
if option(3)=='t'
   zsl=plt3d(rmovie,'T',Zz,Zz(length(Zz)),observation);
else
   zsl=plt3d(rmovie,'Z',Zz,Zz(length(Zz)),observation);
end

if rmovie=='y'
   TLen=length(zsl);
end
   
if (plot_type == 'e') & (plot_option == 'lin')
   resp = abs(resp); % removed transpose 26.1.98
elseif (plot_type == 'e') & (plot_option == 'log')
   resp = 20*log10(abs(resp)/n_elem +eps); % removed transp 
elseif (plot_type == 'r') & (plot_option == 'lin')
   resp = real(resp); % removed transp
end


zsize=size(Zz);
% add some text ...

text(0 , 1.0 , Ttxt1);
text(0 , 0.5 , Ttxt2);
tmp = [Ttxt3,vtxt];
text( 0 , 0.0 , tmp);
drawnow;


figure(plotfig);
axes(handle1);
if (TLen > 1)
   M = moviein(TLen,handle1);
end

csys=computer;
if (csys(1:2) == 'PC')
   d='\';
else
   d='/';
end

if (length(visualize)>1)
%   setcomap(plotfig,visualize(5:10));
   colormap(deblank(visualize(5:10)));  % must remove trailing blanks for matlab 6
else
   colormap(hot);
end

% Loop over TLen

for PNo=1:TLen

   if option(1)=='f'
      if (PNo<10)
	 eval(['load ''',usr_resdir,'''',d,'Pict00',num2str(PNo),'.mat'])
	 eval(['resp = Pict00',num2str(PNo),';']);
      elseif (PNo >= 10)&(PNo < 100)
         eval(['load ''',usr_resdir,'''',d,'Pict0',num2str(PNo),'.mat'])
 	 eval(['resp = Pict0',num2str(PNo),';']);
      elseif (PNo >= 100)&(PNo < 1000)
 	 eval(['load ''',usr_resdir,'''',d,'Pict',num2str(Pno),'.mat'])
  	 eval(['resp = Pict',num2str(PNo),';'])
      end

%
% set resp according to plot options
%

      if (plot_type == 'e') & (plot_option == 'lin')
         resp = abs(resp); % removed transp
      elseif (plot_type == 'e') & (plot_option == 'log')
         resp = 20*log10(abs(resp)/n_elem +eps); % removed transp
      elseif (plot_type == 'r') & (plot_option == 'lin')
         resp = real(resp); % removed transp
      end
   end

   if rmovie=='y'
      slcz = zsl(PNo);
   else
      slcz = zsl;
   end

   if (TLen>1)
      fprintf('plot number %3.0f of totally %3.0f plots\n',PNo,TLen);
   end
   vers = version;
   if vers(1)=='5'
    if all(plot_option == 'log')
      slice(Zz,Xx,Yy,clipl(resp,-50),slcz,slcx,slcy);
    else
      slice(Zz,Xx,Yy,resp,slcz,slcx,slcy);
    end
   else
    if all(plot_option == 'log')
      slice(Zz,Xx,Yy,clipl(resp,-50),slcz,slcx,slcy) % matlab 6 doesn't like: ,zsize(2));
    else
      slice(Zz,Xx,Yy,resp,slcz,slcx,slcy) % matlab 6 doesn't like: ,zsize(2));
    end     
     
   end  
   shading flat;

   M(:,PNo) = getframe;

%end of loop over TLen

end

%set axis and view

xlabel(Xtxt)
ylabel(Ytxt)
zlabel(Ztxt)

%
% Play movie
%

if (option(1)=='f') | (rmovie=='y')
   more = 'y'
   while(more =='y')
      clc;
      disp('Movie Options');
      disp('  ');
      n=input('-> Enter number of movie loops [1..100] : ');
     %% if (n == '') % not matlab 6
      if isempty(n)    
         n=10;
      end
      fps=input('-> Enter speed in frames/sec [1..20] : ');
     %% if (fps == '') % not matlab 6
      if isempty(fps) 
         fps= 8;
      end

% make up a vector that plays the movie with a break after each loop.
% To maintain speed, this is done with a vector N to specify the order 
% of the frames
%

      last_picture=size(M,2);
      N=[n,1:last_picture ,ones(1,fps)*last_picture]; % break for 1 sec
      disp('  ');
      disp('Movie is playing ...');
      figure(plotfig);
      movie(M,N,fps);
      disp('  ')
      more = input('-> More movie ??? [y/n] : ','s');
   end


%
% save movie matrix ?
%
%   if (input('-> Do you want to save the movie [y/n] ? ','s') == 'y')
%      [fname,path]=uiputfile('lastone.mvi','Save movie as ...');
%      if ~(fname(1) == 0)
%         eval(['save ',path,fname,' M N']);
%         disp(' ');
%         disp(['Saved movie as ',path,fname]);
%      end
%   end
%   mpgwrite(M,hot,'testmeg.mpeg');

end
